//------------------------------------------------------------------------------
// File       : gig_eth_pcs_pma_sgmii_v11_5_block.v
// Author     : Xilinx Inc.
//------------------------------------------------------------------------------
// (c) Copyright 2004-2009 Xilinx, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES. 
// 
// 
//------------------------------------------------------------------------------
// Description: This Core Block Level wrapper instantiates IOB flip-flops
//              and input/output buffers on the TBI interface of the
//              core.
//
//              All of the clock circuitry required for a single
//              instance of the core is included.
//
//              The SGMII adaptation module is provided to convert
//              between 1Gbps and 10/100 Mbps rates.  This is connected
//              to the MAC side of the core to provide a GMII style
//              interface.  When the core is running at 1Gbps speeds,
//              the GMII (8-bitdata pathway) is used at a clock
//              frequency of 125MHz.  When the core is running at
//              100Mbps, a clock frequency of 12.5MHz is used.  When
//              running at 100Mbps speeds, a clock frequency of 1.25MHz
//              is used.
//
//           -----------------------------------------------------
//           |                 Core Block Level wrapper          |
//           |                                                   |
//           |                  -----------------------          |
//           |                  |        Core         |          |
//           |                  |                     |          |
//           |                  |                     |          |
//           |    ---------     |                     |          |
//       ----|    |       |     | GMII            TBI |--------------->
//           |    | SGMII |     | Tx              Tx  |          |
//           ---->| Adapt |---->|                     |          |
//           |    | Module|     |                     |    TBI   |
//           |    |       |     |                     |    IOBs  |
//           |    |       |     |                     |          |
//           |    |       |     |                     |          |
//           |    |       |     | GMII            TBI |          |
//       <---|    |       |     | Rx              Rx  |<---------------
//           |    |       |     |                     |          |
//           -----|       |<----|                     |          |
//           |    |       |     |                     |          |
//           |    ---------     ----------------------           |
//           |                                                   |
//       -----------------------------------------------------
//


`timescale 1 ps/1 ps

//------------------------------------------------------------------------------
// The module declaration for the Core Block Level wrapper.
//------------------------------------------------------------------------------

module gig_eth_pcs_pma_sgmii_v11_5_block
   (
      // TBI Interface (PCS <=> PMA)
      //----------------------------

      input            gtx_clk,               // Reference 125MHz clock.
      output reg [9:0] tx_code_group,         // The ten bit parallel transmit data presented to the PMA.
      output           pma_tx_clk,            // The 125 MHz transmit code group clock.
      output           loc_ref,               // Causes the SERDES clock recovery to lock to PMA_TX_CLK.
      output reg       ewrap,                 // This instructs the PMA to loopback the data.
      input      [9:0] rx_code_group,         // The ten bit parallel received data from the PMA.
      input            pma_rx_clk0,           // 62.5MHz rx clock used to reclock even code-groups.
      output reg       en_cdet,               // Enables the PMA sublayer to perform the code-group alignment function on a comma.

      // GMII Interface (client MAC <=> PCS)
      //------------------------------------

      output           sgmii_clk_r,           // Clock for client MAC (125Mhz, 12.5MHz or 1.25MHz).
      output           sgmii_clk_f,           // Clock for client MAC (125Mhz, 12.5MHz or 1.25MHz).
      output           sgmii_clk_en,          // Clock enable for client MAC

      input      [7:0] gmii_txd,              // Transmit data from client MAC.
      input            gmii_tx_en,            // Transmit control signal from client MAC.
      input            gmii_tx_er,            // Transmit control signal from client MAC.
      output     [7:0] gmii_rxd,              // Received Data to client MAC.
      output           gmii_rx_dv,            // Received control signal to client MAC.
      output           gmii_rx_er,            // Received control signal to client MAC.
      output           gmii_isolate,          // Tristate control to electrically isolate GMII.

      // Management: MDIO Interface
      //---------------------------

      input            mdc,                   // Management Data Clock
      input            mdio_i,                // Management Data In
      output           mdio_o,                // Management Data Out
      output           mdio_t,                // Management Data Tristate
      input      [4:0] phyad,                 // Port address for MDIO.
      input      [4:0] configuration_vector,  // Alternative to MDIO interface.
      input            configuration_valid,   // Validation signal for Config vector

      output           an_interrupt,          // Interrupt to processor to signal that Auto-Negotiation has completed
      input     [15:0] an_adv_config_vector, // Alternate interface to program REG4 (AN ADV)
      input            an_adv_config_val,    // Validation signal for AN ADV
      input            an_restart_config,    // Alternate signal to modify AN restart bit in REG0
      // Dynamic switching between 1000BASE-X and SGMII standards
      //---------------------------------------------------------
      input      [8:0] link_timer_basex,      // Programmable Auto-Negotiation Link Timer Control for 1000BASE-X standard
      input      [8:0] link_timer_sgmii,      // Programmable Auto-Negotiation Link Timer Control for SGMII standard
      input            basex_or_sgmii,        // Reset default for 1000BASE-X (0) or SGMII standard (1)

      // Speed Control
      //--------------
      input            speed_is_10_100,       // Core should operate at either 10Mbps or 100Mbps speeds
      input            speed_is_100,          // Core should operate at 100Mbps speed

      // General IO's
      //-------------
      output    [15:0]  status_vector,         // Synchronisation status.
      input            reset,                 // Asynchronous reset for entire core.
      input            signal_detect          // Input from PMD to indicate presence of optical input.
   );



   //---------------------------------------------------------------------------
   // internal signals used in this Block level wrapper.
   //---------------------------------------------------------------------------

  // GMII signals routed between core and SGMII Adaptation Module
   wire  [7:0]  gmii_txd_int;                 // internal gmii_txd signal.
   wire         gmii_tx_en_int;               // internal gmii_tx_en signal.
   wire         gmii_tx_er_int;               // internal gmii_tx_er signal.
   wire  [7:0]  gmii_rxd_int;                 // internal gmii_rxd signal.
   wire         gmii_rx_dv_int;               // internal gmii_rx_dv signal.
   wire         gmii_rx_er_int;               // internal gmii_rx_er signal.

   // TBI signals
   wire [9:0]   tx_code_group_int;            // internal tx_code_group signal.
   wire [9:0]   rx_code_group_delay;          // rx_code_group is routed through delay elements
   wire [9:0]   rx_code_group0_iff;           // rx_code_group registered in IOBs on PMA_RX_CLK0.
   wire [9:0]   rx_code_group1_iff;           // rx_code_group registered in IOBs on PMA_RX_CLK1.
   reg  [9:0]   rx_code_group0_reg;           // rx_code_group0 reclocked onto regional clock.
   reg  [9:0]   rx_code_group1_reg;           // rx_code_group1 reclocked onto regional clock.
   wire         not_gtx_clk;                  // Route gtx_clk_bufg through an inverter.
   wire         pma_rx_clk0_bufio;            // pma_rx_clk0 routed through a BUFIO2.
   wire         pma_rx_clk0_bufg;             // pma_rx_clk0 routed through a BUFG.
   wire         pma_rx_clk0_to_bufg;          // Route from BUFIO2 to BUFG
   wire         not_pma_rx_clk0_bufio;        // Inverted pma_rx_clk0_bufio.
   wire         ewrap_int;                    // internal ewrap signal.
   wire         en_cdet_int;                  // internal en_cdet signal.



   //---------------------------------------------------------------------------
   // Component Instantiation for the SGMII adaptation module
   //---------------------------------------------------------------------------

   gig_eth_pcs_pma_sgmii_v11_5_sgmii_adapt sgmii_logic (
      .reset                 (reset),
      .clk125m               (gtx_clk),
      .sgmii_clk_r           (sgmii_clk_r),
      .sgmii_clk_f           (sgmii_clk_f),
      .sgmii_clk_en          (sgmii_clk_en),
      .gmii_txd_in           (gmii_txd),
      .gmii_tx_en_in         (gmii_tx_en),
      .gmii_tx_er_in         (gmii_tx_er),
      .gmii_rxd_in           (gmii_rxd_int),
      .gmii_rx_dv_in         (gmii_rx_dv_int),
      .gmii_rx_er_in         (gmii_rx_er_int),
      .gmii_txd_out          (gmii_txd_int),
      .gmii_tx_en_out        (gmii_tx_en_int),
      .gmii_tx_er_out        (gmii_tx_er_int),
      .gmii_rxd_out          (gmii_rxd),
      .gmii_rx_dv_out        (gmii_rx_dv),
      .gmii_rx_er_out        (gmii_rx_er),
      .speed_is_10_100       (speed_is_10_100),
      .speed_is_100          (speed_is_100)
      );


  //----------------------------------------------------------------------------
  // Instantiate the core
  //----------------------------------------------------------------------------
  gig_eth_pcs_pma_sgmii_v11_5 gig_eth_pcs_pma_core
     (
      .gtx_clk               (gtx_clk),
      .tx_code_group         (tx_code_group_int),
      .loc_ref               (loc_ref),
      .ewrap                 (ewrap_int),
      .rx_code_group0        (rx_code_group0_reg),
      .rx_code_group1        (rx_code_group1_reg),
      .pma_rx_clk0           (pma_rx_clk0_bufg),
      .pma_rx_clk1           (pma_rx_clk0_bufg),
      .en_cdet               (en_cdet_int),
      .gmii_txd              (gmii_txd_int),
      .gmii_tx_en            (gmii_tx_en_int),
      .gmii_tx_er            (gmii_tx_er_int),
      .gmii_rxd              (gmii_rxd_int),
      .gmii_rx_dv            (gmii_rx_dv_int),
      .gmii_rx_er            (gmii_rx_er_int),
      .gmii_isolate          (gmii_isolate),
      .mdc                   (mdc),
      .mdio_in               (mdio_i),
      .mdio_out              (mdio_o),
      .mdio_tri              (mdio_t),
      .phyad                 (phyad),
      .configuration_vector  (configuration_vector),
      .configuration_valid   (configuration_valid),
      .an_interrupt          (an_interrupt),
      .an_adv_config_vector  (an_adv_config_vector),
      .an_adv_config_val     (an_adv_config_val),
      .an_restart_config     (an_restart_config),
      .link_timer_basex      (link_timer_basex),
      .link_timer_sgmii      (link_timer_sgmii),
      .basex_or_sgmii        (basex_or_sgmii),
      .status_vector         (status_vector),
      .reset                 (reset),
      .signal_detect         (signal_detect)

      );



  //----------------------------------------------------------------------------
  // TBI Transmitter Clock Management : derive pma_tx_clk from gtx_clk
  //----------------------------------------------------------------------------

  //  gtx_clk is inverted. This inversion takes place locally in the IOB.
  INV invert_gtx_clk (
     .I   (gtx_clk),
     .O   (not_gtx_clk)
  );


  // Instantiate a DDR output register.  This is a good way to drive
  // pma_tx_clk since the clock-to-PAD delay will be the same as that
  // for data driven from IOB Ouput flip-flops eg tx_code_group[9:0].
  // This is set to produce an inverted clock w.r.t. gtx_clk so
  // that the rising edge is centralised within the tx_code_group[9:0]
  // valid window.
  
  ODDR2 pma_tx_clk_ddr_iob (
     .Q   (pma_tx_clk),
     .C0  (gtx_clk),
     .C1  (not_gtx_clk),
     .CE  (1'b1),
     .D0  (1'b0),
     .D1  (1'b1),
     .R   (1'b0),
     .S   (1'b0)
    );        


  //----------------------------------------------------------------------------
  // TBI Transmitter Logic : drive TX signals through IOBs onto TBI
  //----------------------------------------------------------------------------

  // Infer IOB Output flip-flops.
  always @ (posedge gtx_clk)
  begin
     tx_code_group <= tx_code_group_int;
  end


  //----------------------------------------------------------------------------
  // TBI Control Signals : drive TBI control through IOBs signals onto
  // TBI
  //----------------------------------------------------------------------------

  // Infer IOB Output flip-flops.
  always @ (posedge gtx_clk)
  begin
      ewrap   <= ewrap_int;
      en_cdet <= en_cdet_int;
  end


  //----------------------------------------------------------------------------
  // TBI Receiver Clock Management : receive PMA_RX_CLK0/PMA_RX_CLK1
  // through IOBs from TBI
  //----------------------------------------------------------------------------

   // Route pma_rx_clk0 from PAD through a BUFIO2 Buffer
   BUFIO2 # (
     .USE_DOUBLER  ("TRUE"),
     .DIVIDE       (2),
     .DIVIDE_BYPASS("FALSE"),
     .I_INVERT     ("FALSE")
   ) receive_pma_rx_clk0 (
     .DIVCLK       (pma_rx_clk0_to_bufg),
     .I            (pma_rx_clk0),
     .IOCLK        (pma_rx_clk0_bufio),
     .SERDESSTROBE ()
   );


   // Route pma_rx_clk0 from PAD through a BUFIO2 Buffer to produce an inverted
   // clock
   BUFIO2 # (
     .I_INVERT     ("TRUE")
   ) receive_pma_rx_clk0_inv (
     .DIVCLK       (),
     .I            (pma_rx_clk0),
     .IOCLK        (not_pma_rx_clk0_bufio),
     .SERDESSTROBE ()
   );


   // Route pma_rx_clk0 through a BUFG onto global clock routing
   BUFG drive_pma_rx_clk0 (
     .I            (pma_rx_clk0_to_bufg),
     .O            (pma_rx_clk0_bufg)
   );


  //----------------------------------------------------------------------------
  // TBI Receiver Logic : receive RX signals through IOBs from TBI
  //----------------------------------------------------------------------------

  genvar i;
  generate for (i=0; i<10; i=i+1)
    begin : tbi_rx_data_bus

      // An IODELAY2 is added to the data inputs on Spartan-6 devices.
      // The tap delay values can be adjusted for PCB routing deskew.

      IODELAY2 # (
          .IDELAY_TYPE  ("FIXED"),
          .DELAY_SRC    ("IDATAIN")

      ) delay_tbi_rx_data (
          .BUSY         (),
          .DATAOUT      (rx_code_group_delay[i]),
          .DATAOUT2     (),
          .DOUT         (),
          .TOUT         (),
          .CAL          (1'b0),
          .CE           (1'b0),
          .CLK          (1'b0),
          .IDATAIN      (rx_code_group[i]),
          .INC          (1'b0),
          .IOCLK0       (1'b0),
          .IOCLK1       (1'b0),
          .ODATAIN      (1'b0),
          .RST          (1'b0),
          .T            (1'b1)
      );


      // Instantiate Double-Data_Rate Registers synchronous to pma_rx_clk0.
      IDDR2 # (
          .DDR_ALIGNMENT ("C0")

      ) rx_code_group_ddr (
          .Q0           (rx_code_group0_iff[i]),
          .Q1           (rx_code_group1_iff[i]),
          .C0           (pma_rx_clk0_bufio),
          .C1           (not_pma_rx_clk0_bufio),
          .CE           (1'b1),
          .D            (rx_code_group_delay[i]),
          .R            (1'b0),
          .S            (1'b0)
      );         

    end
  endgenerate


   // Reclock onto global clock routing

   always @ (posedge pma_rx_clk0_bufg)
   begin
      rx_code_group0_reg  <= rx_code_group0_iff;
      rx_code_group1_reg  <= rx_code_group1_iff;
   end



endmodule // gig_eth_pcs_pma_sgmii_v11_5_block

