//------------------------------------------------------------------------------
// File       : gig_eth_pcs_pma_sgmii_v11_5_example_design.v
// Author     : Xilinx Inc.
//------------------------------------------------------------------------------
// (c) Copyright 2002-2008 Xilinx, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES. 
// 
// 
//------------------------------------------------------------------------------
// Description: This is the top level vhdl example design for the
//              Ethernet SGMII core.
//
//              This design example instantiates IOB flip-flops
//              and input/output buffers on the GMII.
//
//              Additionally, the I/O of the GMII-style interface is
//              provided with IOB flip-flops (infered) which enables
//              this example design to be implemented using the Xilinx
//              tools.
//
//           * Please refer to the Getting Started User Guide for
//             details of the example design file hierarchy.


`timescale 1 ps/1 ps

//------------------------------------------------------------------------------
// The module declaration for the example design
//------------------------------------------------------------------------------

module gig_eth_pcs_pma_sgmii_v11_5_example_design
   (

      // TBI Interface (PCS <=> PMA)
      //----------------------------

      input            gtx_clk,               // Reference 125MHz clock.
      output     [9:0] tx_code_group,         // The ten bit parallel transmit data presented to the PMA.
      output           pma_tx_clk,            // The 125 MHz transmit code group clock.
      output           loc_ref,               // Causes the SERDES clock recovery to lock to PMA_TX_CLK.
      output           ewrap,                 // This instructs the PMA to loopback the data.
      input      [9:0] rx_code_group,         // The ten bit parallel received data from the PMA.
      input            pma_rx_clk0,           // 62.5MHz rx clock used to reclock even code-groups.
      output           en_cdet,               // Enables the PMA sublayer to perform the code-group alignment function on a comma.

      // GMII Interface (client MAC <=> PCS)
      //------------------------------------
      output           sgmii_clk,             // Clock for client MAC (125Mhz, 12.5MHz or 1.25MHz).
      input      [7:0] gmii_txd,              // Transmit data from client MAC.
      input            gmii_tx_en,            // Transmit control signal from client MAC.
      input            gmii_tx_er,            // Transmit control signal from client MAC.
      output reg [7:0] gmii_rxd,              // Received Data to client MAC.
      output reg       gmii_rx_dv,            // Received control signal to client MAC.
      output reg       gmii_rx_er,            // Received control signal to client MAC.

      // Management: MDIO Interface
      //---------------------------

      input            mdc,                   // Management Data Clock
      input            mdio_i,                // Management Data In
      output           mdio_o,                // Management Data Out
      output           mdio_t,                // Management Data Tristate
      input      [4:0] phyad,                 // Port address for MDIO.
      input            configuration_valid,   // Validation signal for Config vector
      input      [4:0] configuration_vector,  // Alternative to MDIO interface.

      output           an_interrupt,          // Interrupt to processor to signal that Auto-Negotiation has completed
      input [15:0]     an_adv_config_vector,  // Alternate interface to program REG4 (AN ADV)
      input            an_adv_config_val,     // Validation signal for AN ADV
      input            an_restart_config,     // Alternate signal to modify AN restart bit in REG0
      // Dynamic switching between 1000BASE-X and SGMII standards
      //---------------------------------------------------------
      input      [8:0] link_timer_basex,      // Programmable Auto-Negotiation Link Timer Control for 1000BASE-X standard
      input      [8:0] link_timer_sgmii,      // Programmable Auto-Negotiation Link Timer Control for SGMII standard
      input            basex_or_sgmii,        // Reset default for 1000BASE-X (0) or SGMII standard (1)

      // Speed Control
      //--------------
      input            speed_is_10_100,       // Core should operate at either 10Mbps or 100Mbps speeds
      input            speed_is_100,          // Core should operate at 100Mbps speed

      // General IO's
      //-------------
      output reg [15:0] status_vector,         // Synchronisation status.
      input            reset,                 // Asynchronous reset for entire core.
      input            signal_detect          // Input from PMD to indicate presence of optical input.

   );



  //----------------------------------------------------------------------------
  // internal signals used in this top level example design.
  //----------------------------------------------------------------------------

   // Clock Signals
   wire         gtx_clk_bufg;                 // gtx_clk routed through a BUFG: onto Global Clock Routing.
   wire         not_gtx_clk;                  // Route gtx_clk through an inverter.

   // clock generation signals for SGMII clock
   wire         sgmii_clk_r;                  // Clock to client MAC (125MHz, 12.5MHz or 1.25MHz) (to rising edge DDR).
   wire         sgmii_clk_f;                  // Clock to client MAC (125MHz, 12.5MHz or 1.25MHz) (to falling edge DDR).

   // GMII signals
   wire         gmii_isolate;                 // internal gmii_isolate signal.
   reg   [7:0]  gmii_txd_int;                 // internal gmii_txd signal.
   reg          gmii_tx_en_int;               // internal gmii_tx_en signal.
   reg          gmii_tx_er_int;               // internal gmii_tx_er signal.
   wire  [7:0]  gmii_rxd_int;                 // internal gmii_rxd signal.
   wire         gmii_rx_dv_int;               // internal gmii_rx_dv signal.
   wire         gmii_rx_er_int;               // internal gmii_rx_er signal.



   // Extra registers to ease IOB placement
   wire  [15:0]  status_vector_int;


  //----------------------------------------------------------------------------
  // gtx_clk Clock Management.  This is the 125MHz reference source
  // clock for the core.
  //----------------------------------------------------------------------------

  // Route gtx_clk through a BUFG (onto Global Clock Routing)
  BUFG  bufg_gtx_clk (
    .I           (gtx_clk),
    .O           (gtx_clk_bufg)
  );



  //----------------------------------------------------------------------------
  // Instantiate the Core Block (core wrapper).
  //----------------------------------------------------------------------------
  gig_eth_pcs_pma_sgmii_v11_5_block core_wrapper
    (
      .gtx_clk               (gtx_clk_bufg),
      .tx_code_group         (tx_code_group),
      .pma_tx_clk            (pma_tx_clk),
      .loc_ref               (loc_ref),
      .ewrap                 (ewrap),
      .rx_code_group         (rx_code_group),
      .pma_rx_clk0           (pma_rx_clk0 ),
      .en_cdet               (en_cdet),
      .sgmii_clk_r           (sgmii_clk_r),
      .sgmii_clk_f           (sgmii_clk_f),
      .sgmii_clk_en          (),
      .gmii_txd              (gmii_txd_int),
      .gmii_tx_en            (gmii_tx_en_int),
      .gmii_tx_er            (gmii_tx_er_int),
      .gmii_rxd              (gmii_rxd_int),
      .gmii_rx_dv            (gmii_rx_dv_int),
      .gmii_rx_er            (gmii_rx_er_int),
      .gmii_isolate          (gmii_isolate),
      .mdc                   (mdc),
      .mdio_i                (mdio_i),
      .mdio_o                (mdio_o),
      .mdio_t                (mdio_t),
      .phyad                 (phyad),
      .configuration_vector  (configuration_vector),
      .configuration_valid   (configuration_valid),
      .an_interrupt          (an_interrupt),
      .an_adv_config_vector (an_adv_config_vector),
      .an_adv_config_val    (an_adv_config_val),
      .an_restart_config    (an_restart_config),
      .link_timer_basex      (link_timer_basex),
      .link_timer_sgmii      (link_timer_sgmii),
      .basex_or_sgmii        (basex_or_sgmii),
      .speed_is_10_100       (speed_is_10_100),
      .speed_is_100          (speed_is_100),
      .status_vector         (status_vector_int),
      .reset                 (reset),
      .signal_detect         (signal_detect)
    );


   //---------------------------------------------------------------------------
   // GMII transmitter data logic
   //---------------------------------------------------------------------------


   // Drive input GMII signals through IOB input flip-flops (inferred).
   always @ (posedge gtx_clk_bufg)
     begin
         gmii_txd_int    <= gmii_txd;
         gmii_tx_en_int  <= gmii_tx_en;
         gmii_tx_er_int  <= gmii_tx_er;
     end



   //---------------------------------------------------------------------------
   // SGMII clock logic
   //---------------------------------------------------------------------------

   //  gtx_clk_bufg is inverted. This inversion takes place in the IOB
   INV invert_gtx_clk_bufg 
   (
      .I  (gtx_clk_bufg),
      .O  (not_gtx_clk)
   );


   // This instantiates a DDR output register.  This is a nice way to
   // drive the output clock since the clock-to-PAD delay will the
   // same as that of data driven from an IOB Ouput flip-flop.  This is
   // set to produce an inverted clock w.r.t. gtx_clk_bufg so that clock
   // rising edge appears in the centre of GMII data.
   ODDR2 sgclk_ddr_iob
   (
      .Q  (sgmii_clk),
      .C0 (gtx_clk_bufg),
      .C1 (not_gtx_clk),
      .CE (1'b1),
      .D0 (sgmii_clk_r),
      .D1 (sgmii_clk_f),
      .R  (1'b0),
      .S  (1'b0)
   );



   //---------------------------------------------------------------------------
   // GMII receiver data logic
   //---------------------------------------------------------------------------


   // Drive input GMII signals through IOB output flip-flops (inferred).
   always @ (posedge gtx_clk_bufg)
     begin
         gmii_rxd    <= gmii_rxd_int;
         gmii_rx_dv  <= gmii_rx_dv_int;
         gmii_rx_er  <= gmii_rx_er_int;
     end




   //---------------------------------------------------------------------------
   // Extra registers to ease IOB placement
   //---------------------------------------------------------------------------
   always @ (posedge gtx_clk_bufg)
     begin
         status_vector <= status_vector_int;
     end


endmodule // gig_eth_pcs_pma_sgmii_v11_5_example_design

