/*******************************************************************************
*                                                                              *
* System      : BELLE-II ARICH merger module                                   *
* Module      : LOC_REG                                                        *
* Version     : v 1.0.0 2013/12/02                                             *
*                                                                              *
* Description : Slow control; registers                                        *
*                                                                              *
* Designer    : Tomohisa Uchida                                                *
*                                                                              *
*                Copyright (c) 2013 Tomohisa Uchida                            *
*                All rights reserved                                           *
*                                                                              *
*******************************************************************************/

module LOC_REG(
    CLK                 ,   // in   : Clock
    RST                 ,   // in   : System reset
    // System monitor
    SMON_DADDR          ,   // out  : address[6:0]
    SMON_DEN            ,   // out  : Device enable
    SMON_DWE            ,   // out  : Write enable
    SMON_DWD            ,   // out  : Write data[15:0]
    SMON_DRD            ,   // in   : Read data[15:0]
    SMON_RDY            ,   // in   : Ready
    // SiTCP RBCP I/F
    LOC_ADDR            ,   // in   : Address[31:0]
    LOC_WD              ,   // in   : Data[7:0]
    LOC_WE              ,   // in   : Write enable
    LOC_RE              ,   // in   : Read enable
    LOC_ACK             ,   // out  : Access acknowledge
    LOC_RD              ,   // out  : Read data[7:0]
    // JTAG I/F
    JTAG_ENB            ,   // out  : JTAG I/F enable for each FEEs
    JTAG_FULL           ,   // in   : JTAG FIFO full flag
	 JTAG_PFULL           ,   // in   : JTAG FIFO Prog. full flag
    JTAG_WE             ,   // out  : Write enable for JTAG data
    JTAG_WD             ,   // out  : Write data
	 JTAG_RD_ENB			,
	 JTAG_RD_ENB_OFF		,
	 JTAG_FC					,
	 CLK200M_EN				,
    // Arich registers
    FEE_DATA_WIDTH4_8       // out  : Data width of FEE data (0:4-bit, 1:8-bit)
);

//-------- Input/Output -------------
    input           CLK             ;
    input           RST             ;

    output  [6:0]   SMON_DADDR      ;
    output          SMON_DEN        ;
    output          SMON_DWE        ;
    output  [15:0]  SMON_DWD        ;
    input   [15:0]  SMON_DRD        ;
    input           SMON_RDY        ;

    input   [31:0]  LOC_ADDR        ;
    input   [7:0]   LOC_WD          ;
    input           LOC_WE          ;
    input           LOC_RE          ;
    output          LOC_ACK         ;
    output  [7:0]   LOC_RD          ;
    
    output  [5:0]   JTAG_ENB        ;
    input           JTAG_FULL       ;
	 input           JTAG_PFULL       ;
    output          JTAG_WE         ;
    output  [7:0]   JTAG_WD         ;
	 output			  JTAG_RD_ENB		;
	 input			  JTAG_RD_ENB_OFF	;
	 output	[7:0]	  JTAG_FC			;

    output          FEE_DATA_WIDTH4_8   ;
	 output			  CLK200M_EN		;

//------------------------------------------------------------------------------
//  Input buffer
//------------------------------------------------------------------------------
    reg             regCs           ;
    reg             smonCs          ;
    reg             jtagCs          ;
    reg     [10:0]  irAddr          ;
    reg             irWe            ;
    reg             irRe            ;
    reg     [7:0]   irWd            ;
	 reg		[3:0]	  FIFO_SEL			;

    always @(posedge CLK) begin
        regCs       <= (LOC_ADDR[31:8]==24'h0);
        smonCs      <= (LOC_ADDR[31:8]==24'h1);
        jtagCs      <= (LOC_ADDR[31:8]==24'h2);
		  
		  FIFO_SEL[0] <= (LOC_ADDR[31:8]==24'h2) & (LOC_ADDR[1:0]==2'd0);
		  FIFO_SEL[1] <= (LOC_ADDR[31:8]==24'h2) & (LOC_ADDR[1:0]==2'd1);
		  FIFO_SEL[2] <= (LOC_ADDR[31:8]==24'h2) & (LOC_ADDR[1:0]==2'd2);
		  FIFO_SEL[3] <= (LOC_ADDR[31:8]==24'h2) & (LOC_ADDR[1:0]==2'd3);
		  
        irAddr[10:0]    <= LOC_ADDR[10:0];
        irWe            <= LOC_WE;
        irRe            <= LOC_RE;
        irWd[7:0]       <= LOC_WD[7:0];
    end

//------------------------------------------------------------------------------
//  System monitor
//------------------------------------------------------------------------------
    reg             jtagWe          ;
	 reg		[3:0]   jtagEnb			;
    reg             smonDs          ;
    reg     [1:0]   smonRe          ;
    reg     [1:0]   smonWe          ;
    reg     [15:0]  smonWd          ;

    always @(posedge CLK) begin
        jtagWe      <= jtagCs & irWe;
		  jtagEnb[3:0] <= FIFO_SEL[3:0];

        smonDs      <= smonCs & (~irAddr[0] & irRe | irAddr[0] & irWe);

        smonRe[0]   <= smonCs & ~irAddr[0] & irRe;
        smonRe[1]   <= smonCs &  irAddr[0] & irRe;

        smonWe[0]   <= smonCs & ~irAddr[0] & irWe;
        smonWe[1]   <= smonCs &  irAddr[0] & irWe;

        if(irWe)begin
            smonWd[15:8]    <= (~irAddr[0] ? irWd[7:0] : smonWd[15:8]);
            smonWd[7:0]     <= irWd[7:0];
        end
    end

    assign  SMON_DADDR[6:0] = irAddr[7:1];
    assign  SMON_DEN        = smonDs;
    assign  SMON_DWE        = smonWe[1];
    assign  SMON_DWD[15:0]  = smonWd[15:0];

//------------------------------------------------------------------------------
//  JTAG
//------------------------------------------------------------------------------
    assign  JTAG_WE         = jtagWe;
    assign  JTAG_WD[7:0]    = smonWd[7:0];
	 assign  JTAG_ENB[5:0]   = {2'b00,jtagEnb[3:0]};

    reg             jtagHoldAck ;
    reg             jtagAck     ;

    always @ (posedge CLK or posedge RST) begin
        if(RST)begin
            jtagHoldAck <= 1'b0;
            jtagAck <= 1'b0;
        end else begin
            jtagHoldAck <= JTAG_FULL ? JTAG_WE : jtagHoldAck;
            jtagAck     <= JTAG_FULL ? 1'b0 : JTAG_WE | jtagHoldAck;
        end
    end

//------------------------------------------------------------------------------
//  Registers
//------------------------------------------------------------------------------
//  reg     [7:0]   x00_Reg         ;
//  reg     [7:0]   x01_Reg         ;
//  reg     [7:0]   x02_Reg         ;
//  reg     [7:0]   x03_Reg         ;
//  reg     [7:0]   x04_Reg         ;
    reg     [7:0]   x05_Reg         ;
    reg     [7:0]   x06_Reg         ;
    reg     [7:0]   x07_Reg         ;
    reg     [7:0]   x08_Reg         ;
    reg     [7:0]   x09_Reg         ;
    reg     [7:0]   x0A_Reg         ;
    reg     [7:0]   x0B_Reg         ;
    reg     [7:0]   x0C_Reg         ;
    reg     [7:0]   x0D_Reg         ;
    reg     [7:0]   x0E_Reg         ;
    reg     [7:0]   x0F_Reg         ;

    always @(posedge CLK or posedge RST) begin
        if(RST)begin
//          x00_Reg[7:0]    <= 8'd0;
//          x01_Reg[7:0]    <= 8'd0;
//          x02_Reg[7:0]    <= 8'd0;
//          x03_Reg[7:0]    <= 8'd0;
//          x04_Reg[7:0]    <= 8'd0;
            x05_Reg[7:0]    <= 8'h00;
            x06_Reg[7:0]    <= 8'h00;
            x07_Reg[7:0]    <= 8'h00;
            x08_Reg[7:0]    <= 7'h00;   // JTAG enable
            x09_Reg[7:0]    <= 8'h00;
            x0A_Reg[7:0]    <= 8'h00;
            x0B_Reg[7:0]    <= 8'h00;
            x0C_Reg[7:0]    <= 8'h00;
            x0D_Reg[7:0]    <= 8'h00;
            x0E_Reg[7:0]    <= 8'h00;
            x0F_Reg[7:0]    <= 8'h00;
        end else begin
				x08_Reg[0]    <= (JTAG_RD_ENB_OFF ? 1'b0 : (regCs & (irAddr[3:0]==4'h8) & irWe ? irWd[0] : x08_Reg[0]));
            if(irWe)begin
//              x00_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'h0) ? irWd[7:0] : x00_Reg[7:0]);
//              x01_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'h1) ? irWd[7:0] : x01_Reg[7:0]);
//              x02_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'h2) ? irWd[7:0] : x02_Reg[7:0]);
//              x03_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'h3) ? irWd[7:0] : x03_Reg[7:0]);
//              x04_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'h4) ? irWd[7:0] : x04_Reg[7:0]);
                x05_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'h5) ? irWd[7:0] : x05_Reg[7:0]);
                x06_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'h6) ? irWd[7:0] : x06_Reg[7:0]);
                x07_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'h7) ? irWd[7:0] : x07_Reg[7:0]);
                x08_Reg[7:1]    <= (regCs & (irAddr[3:0]==4'h8) ? irWd[6:1] : x08_Reg[7:1]);
					 //x08_Reg[6:0]    <= (regCs & (irAddr[3:0]==4'h8) ? irWd[6:0] : x08_Reg[6:0]);
                x09_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'h9) ? irWd[7:0] : x09_Reg[7:0]);
                x0A_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'hA) ? irWd[7:0] : x0A_Reg[7:0]);
                x0B_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'hB) ? irWd[7:0] : x0B_Reg[7:0]);
                x0C_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'hC) ? irWd[7:0] : x0C_Reg[7:0]);
                x0D_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'hD) ? irWd[7:0] : x0D_Reg[7:0]);
                x0E_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'hE) ? irWd[7:0] : x0E_Reg[7:0]);
                x0F_Reg[7:0]    <= (regCs & (irAddr[3:0]==4'hF) ? irWd[7:0] : x0F_Reg[7:0]);
            end
        end
    end

//------------------------------------------------------------------------------
//  Read control
//------------------------------------------------------------------------------
    wire    [31:0]  fpgaVer     ;

    assign  fpgaVer[31:0]   = `SYN_DATE;

    reg     [7:0]   rdDataA     ;
    reg     [7:0]   regRv       ;
    reg             regAck      ;
    reg     [1:0]   smonAck     ;
    reg     [15:0]  rdDataI     ;

    always @(posedge CLK) begin
        case(irAddr[3:0])
            4'h0:   rdDataA[7:0]    <= fpgaVer[31:24];
            4'h1:   rdDataA[7:0]    <= fpgaVer[23:16];
            4'h2:   rdDataA[7:0]    <= fpgaVer[15: 8];
            4'h3:   rdDataA[7:0]    <= fpgaVer[ 7: 0];
            4'h4:   rdDataA[7:0]    <= `FPGA_VER;
            4'h5:   rdDataA[7:0]    <= x05_Reg[7:0];
            4'h6:   rdDataA[7:0]    <= x06_Reg[7:0];
            4'h7:   rdDataA[7:0]    <= x07_Reg[7:0];
            4'h8:   rdDataA[7:0]    <= {JTAG_PFULL,x08_Reg[6:0]};
            4'h9:   rdDataA[7:0]    <= x09_Reg[7:0];
            4'hA:   rdDataA[7:0]    <= x0A_Reg[7:0];
            4'hB:   rdDataA[7:0]    <= x0B_Reg[7:0];
            4'hC:   rdDataA[7:0]    <= x0C_Reg[7:0];
            4'hD:   rdDataA[7:0]    <= x0D_Reg[7:0];
            4'hE:   rdDataA[7:0]    <= x0E_Reg[7:0];
            4'hF:   rdDataA[7:0]    <= x0F_Reg[7:0];
				
        endcase

        if(SMON_RDY)begin
            rdDataI[15:0]   <= SMON_DRD[15:0];
        end

        regRv       <= irRe & regCs;
        regAck      <= regCs & (irWe | irRe);

        smonAck[0]  <= smonCs & SMON_RDY;
        smonAck[1]  <= smonRe[1] | smonWe[0];
    end

    reg             orAck       ;
    reg     [7:0]   orRd        ;

    wire            adcAck      ;
    wire    [7:0]   adcRd       ;
	 
    always @(posedge CLK) begin
        orAck       <= regAck | smonAck[1] | smonAck[0] | jtagAck;
        orRd[7:0]   <=  (regRv      ? rdDataA[7:0]  : 8'd0)|
                        (smonAck[0] ? rdDataI[15:8] : 8'd0)|
                        (smonAck[1] ? rdDataI[7:0]  : 8'd0);
    end

    assign  LOC_ACK     = orAck;
    assign  LOC_RD[7:0] = orRd[7:0];

//------------------------------------------------------------------------------
//  Sig. assignment
//------------------------------------------------------------------------------

    assign  FEE_DATA_WIDTH4_8 = x05_Reg[0];
    assign  JTAG_RD_ENB  = x08_Reg[0];
	 assign  CLK200M_EN  = x09_Reg[0];
	 
	 reg		[7:0]		orJTAG_FC;
	 
	 always @(posedge CLK)begin
        orJTAG_FC[7:0] <= x0C_Reg[7:0];
	 end
	 
	 assign	JTAG_FC[7:0] = orJTAG_FC[7:0];

endmodule



