//------------------------------------------------------------------------------
// File       : gig_eth_pcs_pma_v11_5_example_design.v
// Author     : Xilinx Inc.
//------------------------------------------------------------------------------
// (c) Copyright 2009 Xilinx, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES. 
// 
// 
//------------------------------------------------------------------------------
// Description: This is the top level Verilog example design for the
//              Ethernet 1000BASE-X PCS/PMA core.
//
//              This design example instantiates IOB flip-flops
//              on the GMII.
//
//              A Transmitter Elastic Buffer is instantiated on the Tx
//              GMII path to perform clock compenstation between the
//              core and the external MAC driving the Tx GMII.
//
//              This design example can be synthesised.
//
//
//
//    ----------------------------------------------------------------
//    |                             Example Design                   |
//    |                                                              |
//    |             ----------------------------------------------   |
//    |             |           Core Block (wrapper)             |   |
//    |             |                                            |   |
//    |             |   --------------          --------------   |   |
//    |             |   |    Core    |          | tranceiver |   |   |
//    |             |   |            |          |            |   |   |
//    |  ---------  |   |            |          |            |   |   |
//    |  |       |  |   |            |          |            |   |   |
//    |  |  Tx   |  |   |            |          |            |   |   |
//  ---->|Elastic|----->| GMII       |--------->|        TXP |--------->
//    |  |Buffer |  |   | Tx         |          |        TXN |   |   |
//    |  |       |  |   |            |          |            |   |   |
//    |  ---------  |   |            |tranceiver|            |   |   |
//    | GMII        |   |            |    I/F   |            |   |   |
//    | IOBs        |   |            |          |            |   |   |
//    |             |   |            |          |            |   |   |
//    |             |   | GMII       |          |        RXP |   |   |
//  <-------------------| Rx         |<---------|        RXN |<---------
//    |             |   |            |          |            |   |   |
//    |             |   --------------          --------------   |   |
//    |             |                                            |   |
//    |             ----------------------------------------------   |
//    |                                                              |
//    ----------------------------------------------------------------
//
//


`timescale 1 ps/1 ps

//------------------------------------------------------------------------------
// The module declaration for the example design
//------------------------------------------------------------------------------

module gig_eth_pcs_pma_v11_5_example_design
   (
      //------------------------------------------------------------------------
      // Core connected to GTP0
      //------------------------------------------------------------------------

      // GMII Interface
      //---------------
//    input        gmii_tx_clk0,          // Transmit clock from client MAC.
//    output       gmii_rx_clk0,          // Receive clock to client MAC.
      input  [7:0] gmii_txd0,             // Transmit data from client MAC.
      input        gmii_tx_en0,           // Transmit control signal from client MAC.
      input        gmii_tx_er0,           // Transmit control signal from client MAC.
      output [7:0] gmii_rxd0,             // Received Data to client MAC.
      output       gmii_rx_dv0,           // Received control signal to client MAC.
      output       gmii_rx_er0,           // Received control signal to client MAC.

      // Management: MDIO Interface
      //---------------------------
      input        mdc0,                  // Management Data Clock
      input        mdio0_i,               // Management Data In
      output       mdio0_o,               // Management Data Out
      output       mdio0_t,               // Management Data Tristate
      input  [4:0] phyad0,                // Port address for MDIO.
      input  [4:0] configuration_vector0, // Alternative to MDIO interface.
      input        configuration_valid0,  // Validation signal for Config vector

      // General IO's
      //-------------
      input  [8:0] link_timer_value0,     // Programmable Auto-Negotiation Link Timer Control
      output       an_interrupt0,         // Interrupt to processor to signal that Auto-Negotiation has completed
      input [15:0] an_adv_config_vector0, // Alternate interface to program REG4 (AN ADV)
      input        an_adv_config_val0,    // Validation signal for AN ADV
      input        an_restart_config0,    // Alternate signal to modify AN restart bit in REG0
      output [15:0] status_vector0,       // Core status.
      input        reset0,                // Asynchronous reset for entire core.
      input        signal_detect0,        // Input from PMD to indicate presence of optical input.
      output       gmii_isolate0,         // internal gmii_isolate signal.


      //------------------------------------------------------------------------
      // User additional signal
      //------------------------------------------------------------------------

      output       clk125m,               // global clock for user circit.
      output       resetdone0,             // Asynchronous reset done0 & done1 output.

      //------------------------------------------------------------------------
      // Tranceiver interfaces
      //------------------------------------------------------------------------

      input        brefclk_p,             // Differential +ve of reference clock for tranceiver: 125MHz, very high quality
      input        brefclk_n,             // Differential -ve of reference clock for tranceiver: 125MHz, very high quality

      output       txp0,                  // Differential +ve of serial transmission from PMA to PMD.
      output       txn0,                  // Differential -ve of serial transmission from PMA to PMD.
      input        rxp0,                  // Differential +ve for serial reception from PMD to PMA.
      input        rxn0,                  // Differential -ve for serial reception from PMD to PMA.

      output       txp1,                  // Differential +ve of serial transmission from PMA to PMD.
      output       txn1,                  // Differential -ve of serial transmission from PMA to PMD.
      input        rxp1,                  // Differential +ve for serial reception from PMD to PMA.
      input        rxn1                   // Differential -ve for serial reception from PMD to PMA.
   );



   //---------------------------------------------------------------------------
   // internal signals used in this top level example design.
   //---------------------------------------------------------------------------

   // clock/reset generation signals
   wire         clkin;                    // tranceiver 125MHz clock, very high quality.
   wire         userclk2;                 // 125MHz reference clock derived from tranceiver clock circuitry.
// wire         not_userclk2;             // Inverted form of userclk2.
   wire         gtpclkout;                // tranceiver output clock made available to the FPGA fabric.
   wire         gtpclkout_bufio2;         // gtpclkout is routed through a BUFIO2 clock buffer
//   wire         resetdone0;               // Asynchronous reset done0 output.

   // GMII signals for the core connected to GTP0
   wire         gtpreset0;                // System reset for tranceiver.
// wire         gmii_tx_clk_bufio0;       // gmii_tx_clk routed through an BUFIO.
// wire         gmii_tx_clk_bufg0;        // gmii_tx_clk_ibuf routed through a BUFR.
// wire [7:0]   gmii_txd_delay0;          // Internal gmii_txd signal after IDELAY.
// wire         gmii_tx_en_delay0;        // Internal gmii_tx_en signal after IDELAY.
// wire         gmii_tx_er_delay0;        // Internal gmii_tx_er signal after IDELAY.
// wire         gmii_isolate0;            // internal gmii_isolate signal.
// reg  [7:0]   gmii_txd_iff0;            // gmii_txd signal for input IOB flip-flop.
// reg          gmii_tx_en_iff0;          // gmii_tx_en signal for input IOB flip-flop.
// reg          gmii_tx_er_iff0;          // gmii_tx_er signal for input IOB flip-flop.
// reg  [7:0]   gmii_txd_reg0;            // gmii_txd signal reclocked onto regional routing.
// reg          gmii_tx_en_reg0;          // gmii_tx_en signal reclocked onto regional routing.
// reg          gmii_tx_er_reg0;          // gmii_tx_er signal reclocked onto regional routing.
   wire  [7:0]  gmii_txd_fifo0;           // gmii_txd signal after Tx Elastic Buffer.
   wire         gmii_tx_en_fifo0;         // gmii_tx_en signal after Tx Elastic Buffer.
   wire         gmii_tx_er_fifo0;         // gmii_tx_er signal after Tx Elastic Buffer.
   wire  [7:0]  gmii_rxd_int0;            // internal gmii_rxd signal.
   wire         gmii_rx_dv_int0;          // internal gmii_rx_dv signal.
   wire         gmii_rx_er_int0;          // internal gmii_rx_er signal.
// wire         gmii_rx_clk_obuf0;        // gmii_rx_clk  registered in IOBs prior to an OBUF.
// reg   [7:0]  gmii_rxd_obuf0;           // gmii_rxd registered in IOBs prior to an OBUF.
// reg          gmii_rx_dv_obuf0;         // gmii_rx_dv registered in IOBs prior to an OBUF.
// reg          gmii_rx_er_obuf0;         // gmii_rx_er registered in IOBs prior to an OBUF.




   //---------------------------------------------------------------------------
   // Spartan-6 Transceiver Clock Management
   //---------------------------------------------------------------------------

   // NOTE: BREFCLK circuitry for the Transceiver requires the use of a
   // 125MHz differential input clock.  clkin is routed to the tranceiver
   // pair.

   IBUFDS clkingen (
      .I   (brefclk_p),
      .IB  (brefclk_n),
      .O   (clkin)
   );

   // gtpclkout (125MHz) is made avaiable by the tranceiver to the FPGA
   // fabric. This is routed to a BUFIO2 before being placed onto global clock
   // routing where it is then used for tranceiver TXUSRCLK2/RXUSRCLK2 and used
   // to clock all Ethernet core logic.

   // Route through a BUFIO2
   BUFIO2 # (
      .DIVIDE        (1),
      .DIVIDE_BYPASS ("TRUE")
   ) bufio2_clk125m  (
      .DIVCLK        (gtpclkout_bufio2),
      .I             (gtpclkout),
      .IOCLK         (),
      .SERDESSTROBE  ()
   );

   // Route through a BUFG
   BUFG bufg_clk125m (
      .I   (gtpclkout_bufio2),
      .O   (userclk2)
   );

   assign clk125m = userclk2 ;

   //---------------------------------------------------------------------------
   // Spartan-6 Transceiver System Reset
   //---------------------------------------------------------------------------
   // tranceiver 0
   gig_eth_pcs_pma_v11_5_reset_sync gtpreset0_gen (
      .clk       (userclk2),
      .reset_in  (reset0),
      .reset_out (gtpreset0)
   );
   //assign gtpreset0 = reset0 ;

   //---------------------------------------------------------------------------
   // Instantiate the Core Block (core wrapper).
   //---------------------------------------------------------------------------
   gig_eth_pcs_pma_v11_5_block #
   (
      // Simulation attribute: this setting does not affect the hardware
      // It is a Smartmodel setting only.  Setting it to 1 reduces the
      // simulation time required for the GTP to intialise.
      .SIM_GTPRESET_SPEEDUP  (1)
   )
   core_wrapper
   (
      .gtpclkout             (gtpclkout),

      .gtpreset0             (gtpreset0),
      .gmii_txd0             (gmii_txd_fifo0),
      .gmii_tx_en0           (gmii_tx_en_fifo0),
      .gmii_tx_er0           (gmii_tx_er_fifo0),
      .gmii_rxd0             (gmii_rxd_int0),
      .gmii_rx_dv0           (gmii_rx_dv_int0),
      .gmii_rx_er0           (gmii_rx_er_int0),
      .gmii_isolate0         (gmii_isolate0),
      .mdc0                  (mdc0),
      .mdio0_i               (mdio0_i),
      .mdio0_o               (mdio0_o),
      .mdio0_t               (mdio0_t),
      .phyad0                (phyad0),
      .configuration_vector0 (configuration_vector0),
      .configuration_valid0  (configuration_valid0),
      .link_timer_value0     (link_timer_value0),
      .an_interrupt0         (an_interrupt0),
      .an_adv_config_vector0 (an_adv_config_vector0),
      .an_adv_config_val0    (an_adv_config_val0),
      .an_restart_config0    (an_restart_config0),
      .status_vector0        (status_vector0),
      .reset0                (reset0),
      .resetdone0            (resetdone0),
      .signal_detect0        (signal_detect0),

      .clkin                 (clkin),
      .userclk2              (userclk2),

      .txp0                  (txp0),
      .txn0                  (txn0),
      .rxp0                  (rxp0),
      .rxn0                  (rxn0),
		
		.txp1                  (txp1),
      .txn1                  (txn1),
      .rxp1                  (rxp1),
      .rxn1                  (rxn1)

   );

   //---------------------------------------------------------------------------
   // GMII logic for the core connected to GTP0
   //---------------------------------------------------------------------------


   assign gmii_txd_fifo0[7:0] = gmii_txd0[7:0] ;
   assign gmii_tx_en_fifo0    = gmii_tx_en0    ;
   assign gmii_tx_er_fifo0    = gmii_tx_er0    ;

   assign gmii_rxd0[7:0] = gmii_rxd_int0   ;
   assign gmii_rx_dv0    = gmii_rx_dv_int0 ;
   assign gmii_rx_er0    = gmii_rx_er_int0 ;


endmodule // gig_eth_pcs_pma_v11_5_example_design

